# Vefirifed controller in virtual environment

A simple shared virtual environment with an autonomous entity (cleaning robot), controlled by a formally developed verified software. 
This project is a simplified version of the [LIRKIS G-CVE ECA Showcase](https://lgcve-cbot.glitch.me/).

The verified software controller has been developed using [B-Method and Atelier B tool](https://www.atelierb.eu/en/) and translated to JavaScript by the [BKPI compiler](https://hron.fei.tuke.sk/~korecko/FMInGamesExp/resources/BKPICompiler.zip).

Load 
- `index.html` for a common citizen, controlled by you or 
- `cbot.html` for the cleaning robot, controlled automatically by a controller that utilized the verified controller. 

This version is intended for the **Verified Software in Virtual Environments** course of the  [SusTrainable summer school , Rijeka, Croatia, July 2022](https://sustrainable.uniri.hr/).

## Author
Stefan Korecko 

LIRKIS laboratory,
[Department of Computers and Informatics](https://kpi.fei.tuke.sk/en), 
Faculty of Electrical Engineering and Informatics,
Technical University of Košice

## How will we proceed
During the course (tutorial) we will step by step build the functionality of the `cbot-master-controller` component. 
There will be the following steps:
1. The cBot is put to the parking position, facing north.
2. A collection of positions the cBot has to clean is created and the first position is loaded.
3. The verified controller is called to check whether it is safe to perform the desired operation
4. The cBot position and state and also the dirty positions are updated, according to the output of the verified controller. 

   (Now, the cBot moves and cleans, but ignores other citizens).
5. A collection of connected users (citizens) is created and maintained.
6. The sensor array reads positions connected users (citizens) and avoids them. 
7. After the cBot cleans all the dirty positions, it goes to the parking position.
8. After the cBot reaches the parking position, the whole cleaning process restarts.
9. The cBot is changing color according to its state.

## Used Resources
The shared environment has been created using
 - [Networked-A-Frame](https://github.com/haydenjameslee/networked-aframe) [basic example](https://naf-project.glitch.me/), [available for remix on glitch](https://glitch.com/edit/#!/remix/naf-project) and
 - [aframe-environment-component](https://github.com/supermedium/aframe-environment-component) by Supermedium.

## Some notes about A-Frame/Networked-Aframe
 - A-Frame units are meters, 1.0=1m (1 meter) (source: [A-Frame FAQ](https://aframe.io/docs/1.3.0/introduction/faq.html))
 - North = 0°(0rad) yaw (rotation about the Y-axis). The rotation in A-Frame goes counter-clockwise, so West=90°, South=180°, East=270°.
 
## Significant differences between `index.html` and `cbot.html`

 - in `<a-entity id="player"`: `template:#template-avatar-citizen` vs. `template:#template-avatar-cbot`.
 
 Only in `index.html`:
 - `random-color` and `spawn-in-circle` components.
 
 Only in `cbot.html`:
 - `#dirty-location` entities with `cbot-dirty-position` component in the scene,
 - `<a-entity id="player"` without the `wasd-controls` and `look-controls` components but with the `cbot-master-controller` component.

