import CBotControllerCore from "./CBotControllerCore.js";
export default class CBotController {
    //Imported machines
    #cbotcontrollercore;

    // scalar formal parameters
    #maxMsrblDst;
    #safeDstCl;
    #safeDstMov;
    #cleanRange;

    constructor(maxMsrblDst, safeDstCl, safeDstMov, cleanRange) {
        this.#maxMsrblDst = maxMsrblDst;
        this.#safeDstCl = safeDstCl;
        this.#safeDstMov = safeDstMov;
        this.#cleanRange = cleanRange;
        this.#cbotcontrollercore = new CBotControllerCore(this.#maxMsrblDst, this.#safeDstCl, this.#safeDstMov, this.#cleanRange);
        console.log("CBotController initialized");
    }

    updateAndEvaluate(command, p2gAngleInMin, p2gDst, nne, ene, ese, sse, ssw, wsw, wnw, nnw) {
        //Boolean
        let botOn;
        //Boolean
        let botCleaning;
        //Integer
        let angleInMin;
        //Integer
        let dst;
        if (((((((((((((((((((((0 <= command && command <= 3) && 0 <= p2gAngleInMin) && p2gAngleInMin <= 21599) && p2gDst >= 0) && 0 <= nne) && nne <= this.#maxMsrblDst) && 0 <= ene) && ene <= this.#maxMsrblDst) && 0 <= ese) && ese <= this.#maxMsrblDst) && 0 <= sse) && sse <= this.#maxMsrblDst) && 0 <= ssw) && ssw <= this.#maxMsrblDst) && 0 <= wsw) && wsw <= this.#maxMsrblDst) && 0 <= wnw) && wnw <= this.#maxMsrblDst) && 0 <= nnw) && nnw <= this.#maxMsrblDst)) {
            this.#cbotcontrollercore.readGoalPosAndSensors(p2gAngleInMin, p2gDst, nne, ene, ese, sse, ssw, wsw, wnw, nnw);
            switch (command) {
                case 0:
                    this.#cbotcontrollercore.standBy();
                    break;
                case 1:
                    this.#cbotcontrollercore.goToPosAndClean();
                    break;
                case 2:
                    this.#cbotcontrollercore.goToPosAndNoClean();
                    break;
                case 3:
                    if (p2gDst === 0) {
                        this.#cbotcontrollercore.standBy();
                    }
                    else {
                        this.#cbotcontrollercore.goToPosAndNoClean();
                    }
                    break;
                default:
                    this.#cbotcontrollercore.standBy();
            }
            let out = this.#cbotcontrollercore.getInstr4Bot();
            botOn = out[0];
            botCleaning = out[1];
            angleInMin = out[2];
            dst = out[3];
        }
        else {
            botOn = false;
            botCleaning = false;
            angleInMin = 0;
            dst = 0;
        }
        return [botOn, botCleaning, angleInMin, dst]
    }

    get cbotcontrollercore() {
        return this.#cbotcontrollercore;
    }

}
