import ProximSensors from "./ProximSensors.js";

// constants of Implementation: CBotControllerCore_i
//Integer
const EV_STEP = 1;
//Integer
const EV_START = 200;
export default class CBotControllerCore {
    //Imported machines
    #proximsensors;

    // variables of Implementation: CBotControllerCore_i
    #isOn;
    #isCleaning;
    #willBeOn;
    #willClean;
    #goalAngle;
    #goalDst;
    #pos2goAngle;
    #pos2goDst;
    #evDst;
    #evAngle;

    // scalar formal parameters
    #maxMsrblDst;
    #safeDstCl;
    #safeDstMov;
    #cleanRange;

    constructor(maxMsrblDst, safeDstCl, safeDstMov, cleanRange) {
        this.#maxMsrblDst = maxMsrblDst;
        this.#safeDstCl = safeDstCl;
        this.#safeDstMov = safeDstMov;
        this.#cleanRange = cleanRange;
        this.#proximsensors = new ProximSensors(this.#maxMsrblDst);
        this.#isOn = false;
        this.#isCleaning = false;
        this.#willBeOn = false;
        this.#willClean = false;
        this.#goalAngle = 0;
        this.#goalDst = 0;
        this.#pos2goAngle = 0;
        this.#pos2goDst = 0;
        this.#evDst = 0;
        this.#evAngle = 0;
    }


    static get EV_STEP(){
        return EV_STEP;
    }
    static get EV_START(){
        return EV_START;
    }
    getInstr4Bot() {
        //Boolean
        let botOn;
        //Boolean
        let botCleaning;
        //Integer
        let angleInMin;
        //Integer
        let dst;
        this.#isOn = this.#willBeOn;
        this.#isCleaning = this.#willClean;
        botOn = this.#willBeOn;
        botCleaning = this.#willClean;
        angleInMin = this.#pos2goAngle;
        dst = this.#pos2goDst;
        //console.log(`CBotControllerCore.getInstr4Bot()>> if: this.#pos2goAngle=${this.#pos2goAngle},  this.#pos2goDst=${this.#pos2goDst}`);
        return [botOn, botCleaning, angleInMin, dst]
    }

    readGoalPosAndSensors(angleInMin, dst, nne, ene, ese, sse, ssw, wsw, wnw, nnw) {
        if ((((((((((((((((((0 <= angleInMin && angleInMin <= 21599) && 0 <= nne) && nne <= this.#maxMsrblDst) && 0 <= ene) && ene <= this.#maxMsrblDst) && 0 <= ese) && ese <= this.#maxMsrblDst) && 0 <= sse) && sse <= this.#maxMsrblDst) && 0 <= ssw) && ssw <= this.#maxMsrblDst) && 0 <= wsw) && wsw <= this.#maxMsrblDst) && 0 <= wnw) && wnw <= this.#maxMsrblDst) && 0 <= nnw) && nnw <= this.#maxMsrblDst)) {
            this.#goalAngle = angleInMin;
            this.#goalDst = dst;
            this.#proximsensors.readSensors(nne, ene, ese, sse, ssw, wsw, wnw, nnw);
            //console.log(`CBotControllerCore.readGoalPosAndSensors()>> if: this.#goalAngle=${this.#goalAngle},  this.#goalDst=${this.#goalDst}`);
        }
        else {
            this.#goalAngle = 0;
            this.#goalDst = 0;
            this.#proximsensors.readSensors(0, 0, 0, 0, 0, 0, 0, 0);
            //console.log("CBotControllerCore.readGoalPosAndSensors()>> else");
        }
        this.#willBeOn = false;
        this.#willClean = false;
        this.#pos2goDst = 0;
        this.#pos2goAngle = 0;
    }

    standBy() {
        this.#isOn = false;
        this.#isCleaning = false;
        this.#willBeOn = false;
        this.#willClean = false;
        this.#goalAngle = 0;
        this.#goalDst = 0;
        this.#pos2goAngle = 0;
        this.#pos2goDst = 0;
        this.#proximsensors.readSensors(0, 0, 0, 0, 0, 0, 0, 0);
        this.#evDst = 0;
    }

    goToPosAndClean() {
        this.#willBeOn = true;
        {
            //Boolean
            let allClr;
            allClr = this.#proximsensors.allClear(this.#safeDstCl);
            //console.log(`CBotControllerCore.goToPosAndClean()>> allClr=${allClr}`);
            if (this.#isCleaning === true) {
                //console.log(`CBotControllerCore.goToPosAndClean()>> this.#isCleaning === true`);
                if (allClr === false) {
                    this.#pos2goDst = 0;
                    this.#pos2goAngle = 0;
                    this.#evDst = 0;
                    this.#evAngle = 0;
                }
                else {
                    if (this.#goalDst === 0) {
                        this.#pos2goDst = this.#goalDst;
                        this.#pos2goAngle = this.#goalAngle;
                    }
                    else {
                        if (this.#evDst === 0) {
                            {
                                //Boolean
                                let frClr;
                                //Boolean
                                let leftClr;
                                //Boolean
                                let rightClr;
                                frClr = this.#proximsensors.frontClear(this.#goalAngle, this.#safeDstMov);
                                leftClr = this.#proximsensors.frontClear((this.#goalAngle+16200)%21600, this.#safeDstMov);
                                rightClr = this.#proximsensors.frontClear((this.#goalAngle+5400)%21600, this.#safeDstMov);
                                if (frClr === true) {
                                    this.#pos2goDst = this.#goalDst;
                                    this.#pos2goAngle = this.#goalAngle;
                                    this.#evDst = 0;
                                }
                                else {
                                    if (leftClr === true) {
                                        this.#evDst = CBotControllerCore.EV_START;
                                        this.#evAngle = (this.#goalAngle+16200)%21600;
                                        this.#pos2goDst = this.#evDst;
                                        this.#pos2goAngle = this.#evAngle;
                                    }
                                    else if (rightClr === true) {
                                        this.#evDst = CBotControllerCore.EV_START;
                                        this.#evAngle = (this.#goalAngle+5400)%21600;
                                        this.#pos2goDst = this.#evDst;
                                        this.#pos2goAngle = this.#evAngle;
                                    }
                                    else {
                                        this.#pos2goDst = 0;
                                        this.#pos2goAngle = 0;
                                        this.#evDst = 0;
                                        this.#evAngle = 0;
                                    }
                                }
                            }
                        }
                        else {
                            {
                                //Boolean
                                let frEvClr;
                                frEvClr = this.#proximsensors.frontClear(this.#evAngle, this.#safeDstMov);
                                if (frEvClr === true) {
                                    this.#pos2goDst = this.#evDst;
                                    this.#pos2goAngle = this.#evAngle;
                                    if (this.#evDst >= CBotControllerCore.EV_STEP) {
                                        this.#evDst = this.#evDst-CBotControllerCore.EV_STEP;
                                    }
                                }
                                else {
                                    this.#pos2goDst = 0;
                                    this.#pos2goAngle = 0;
                                    this.#evDst = 0;
                                    this.#evAngle = 0;
                                }
                            }
                        }
                    }
                }
                if ((allClr === false || this.#goalDst > this.#cleanRange)) {
                    this.#willClean = false;
                }
                else {
                    this.#willClean = true;
                }
            }
            else {
                //console.log(`CBotControllerCore.goToPosAndClean()>> this.#isCleaning !== true`);
                if ((this.#goalDst <= this.#cleanRange && allClr === true)) {
                    this.#willClean = true;
                }
                else {
                    this.#willClean = false;
                }
                if (this.#goalDst === 0) {
                    this.#pos2goDst = this.#goalDst;
                    this.#pos2goAngle = this.#goalAngle;
                }
                else {
                    //console.log(`CBotControllerCore.goToPosAndClean()>> not in clean range`);
                    if (this.#evDst === 0) {
                        {
                            //Boolean
                            let frClr;
                            //Boolean
                            let leftClr;
                            //Boolean
                            let rightClr;
                            frClr = this.#proximsensors.frontClear(this.#goalAngle, this.#safeDstMov);
                            leftClr = this.#proximsensors.frontClear((this.#goalAngle+16200)%21600, this.#safeDstMov);
                            rightClr = this.#proximsensors.frontClear((this.#goalAngle+5400)%21600, this.#safeDstMov);
                            //console.log("CBotControllerCore.goToPosAndClean()>> frClr="+frClr+", leftClr="+leftClr+", rightClr="+rightClr);
                            if (frClr === true) {
                                this.#pos2goDst = this.#goalDst;
                                this.#pos2goAngle = this.#goalAngle;
                                this.#evDst = 0;
                            }
                            else {
                                if (leftClr === true) {
                                    this.#evDst = CBotControllerCore.EV_START;
                                    this.#evAngle = (this.#goalAngle+16200)%21600;
                                    this.#pos2goDst = this.#evDst;
                                    this.#pos2goAngle = this.#evAngle;
                                }
                                else if (rightClr === true) {
                                    this.#evDst = CBotControllerCore.EV_START;
                                    this.#evAngle = (this.#goalAngle+5400)%21600;
                                    this.#pos2goDst = this.#evDst;
                                    this.#pos2goAngle = this.#evAngle;
                                }
                                else {
                                    this.#pos2goDst = 0;
                                    this.#pos2goAngle = 0;
                                    this.#evDst = 0;
                                    this.#evAngle = 0;
                                }
                            }
                        }
                    }
                    else {
                        {
                            //Boolean
                            let frEvClr;
                            frEvClr = this.#proximsensors.frontClear(this.#evAngle, this.#safeDstMov);
                            if (frEvClr === true) {
                                this.#pos2goDst = this.#evDst;
                                this.#pos2goAngle = this.#evAngle;
                                if (this.#evDst >= CBotControllerCore.EV_STEP) {
                                    this.#evDst = this.#evDst-CBotControllerCore.EV_STEP;
                                }
                            }
                            else {
                                this.#pos2goDst = 0;
                                this.#pos2goAngle = 0;
                                this.#evDst = 0;
                                this.#evAngle = 0;
                            }
                        }
                    }
                }
            }
        }
    }

    goToPosAndNoClean() {
        this.#willBeOn = true;
        if (this.#isCleaning === true) {
            this.#willClean = false;
        }
        if (this.#goalDst === 0) {
            this.#pos2goDst = this.#goalDst;
            this.#pos2goAngle = this.#goalAngle;
        }
        else {
            if (this.#evDst === 0) {
                {
                    //Boolean
                    let frClr;
                    //Boolean
                    let leftClr;
                    //Boolean
                    let rightClr;
                    frClr = this.#proximsensors.frontClear(this.#goalAngle, this.#safeDstMov);
                    leftClr = this.#proximsensors.frontClear((this.#goalAngle+16200)%21600, this.#safeDstMov);
                    rightClr = this.#proximsensors.frontClear((this.#goalAngle+5400)%21600, this.#safeDstMov);
                    if (frClr === true) {
                        this.#pos2goDst = this.#goalDst;
                        this.#pos2goAngle = this.#goalAngle;
                        this.#evDst = 0;
                    }
                    else {
                        if (leftClr === true) {
                            this.#evDst = CBotControllerCore.EV_START;
                            this.#evAngle = (this.#goalAngle+16200)%21600;
                            this.#pos2goDst = this.#evDst;
                            this.#pos2goAngle = this.#evAngle;
                        }
                        else if (rightClr === true) {
                            this.#evDst = CBotControllerCore.EV_START;
                            this.#evAngle = (this.#goalAngle+5400)%21600;
                            this.#pos2goDst = this.#evDst;
                            this.#pos2goAngle = this.#evAngle;
                        }
                        else {
                            this.#pos2goDst = 0;
                            this.#pos2goAngle = 0;
                            this.#evDst = 0;
                            this.#evAngle = 0;
                        }
                    }
                }
            }
            else {
                {
                    //Boolean
                    let frEvClr;
                    frEvClr = this.#proximsensors.frontClear(this.#evAngle, this.#safeDstMov);
                    if (frEvClr === true) {
                        this.#pos2goDst = this.#evDst;
                        this.#pos2goAngle = this.#evAngle;
                        if (this.#evDst >= CBotControllerCore.EV_STEP) {
                            this.#evDst = this.#evDst-CBotControllerCore.EV_STEP;
                        }
                    }
                    else {
                        this.#pos2goDst = 0;
                        this.#pos2goAngle = 0;
                        this.#evDst = 0;
                        this.#evAngle = 0;
                    }
                }
            }
        }
    }

    get proximsensors() {
        return this.#proximsensors;
    }

}
